
from flask_wtf import Form
from wtforms import TextField, TextAreaField
from wtforms.validators import DataRequired, Optional, ValidationError
from sqlalchemy.orm.exc import MultipleResultsFound, NoResultFound

from .utils import parse_headers
from .models import Path

class PathForm(Form):
    location = TextField('location')
    status = TextField('status', default="200")
    headers = TextAreaField('headers')
    response = TextAreaField('response')

    def validate_headers(form, field):
        if field.data:
            try:
                parse_headers(field.data)
            except:
                raise ValidationError('Headers must be one per-line, with a colon between name and value.')

    def validate_location(form, field):
        if not field.data:
            raise ValidationError('Location is required')

        path = Path.query.filter(Path.location == field.data).first()
        if path:
            raise ValidationError('A Path with that URL already exists')



import datetime
import json

from flask import render_template, Blueprint, abort, request, \
        redirect, url_for, render_template
from sqlalchemy.orm.exc import NoResultFound
from pygments import highlight
from pygments.lexers import HttpLexer
from pygments.formatters import HtmlFormatter

from .database import db
from .models import Path, Record
from .forms import PathForm
from .utils import parse_headers

reflectme = Blueprint('reflectme', __name__)

def get_path_instance(location):
    try:
        path = Path.query.filter(Path.location == location).one()
    except NoResultFound:
        abort(404)
    return path

@reflectme.route('/<path:location>', methods=['GET', 'POST', 'DELETE', 'PUT'])
def view_path(location):
    path = get_path_instance(location)

    record = Record()
    record.visited = datetime.datetime.now()
    record.path = path
    record.request_ip = request.remote_addr

    http_request = render_template('http_request.txt', request=request)
    formatted_request = highlight(http_request, HttpLexer(), HtmlFormatter())
    record.request_content = formatted_request

    db.session.add(record)
    db.session.commit()

    headers = parse_headers(path.headers)
    return (path.response, 200, headers)

@reflectme.route('/inspect/<path:location>')
def inspect_path(location):
    path = get_path_instance(location)

    headers = parse_headers(path.headers)
    http_response = render_template('http_response.txt', path=path, headers=headers)
    formatted_response = highlight(http_response, HttpLexer(), HtmlFormatter())

    return render_template('inspect.html', path=path, formatted_response=formatted_response)

@reflectme.route('/', methods=['GET', 'POST'])
def index():
    current_paths = Path.query.all()

    form = PathForm()
    if form.validate_on_submit():
        path = Path()
        form.populate_obj(path)
        db.session.add(path)
        db.session.commit()
        return redirect(url_for('.inspect_path', location=path.location))
    return render_template('index.html', form=form, current_paths=current_paths)




import argparse
import os
from os import path
from appdirs import user_data_dir

from reflectme.app import create_app

def main(argv=None, prog=None, **kwargs):
    """reflectme main entry point"""

    database_location = path.join(user_data_dir('reflectme', 'clavery'), 'database.db')

    parser = argparse.ArgumentParser(description='Create an HTTP server to record and\
                                     respond to requests.')
    parser.add_argument('host', type=str, default='0.0.0.0', nargs='?',
                        help='host to listen on (default: %(default)s)')
    parser.add_argument('port', type=int, default=5000, nargs='?',
                        help='host to listen on (default: %(default)s)')
    parser.add_argument('--database', dest='database', type=str, default=database_location,
                        help='sqlite database location (default: %(default)s)'.format(database_location))
    parser.add_argument('--debug', dest='debug', action='store_true',
                        help='run web server in debug mode')
    parser.add_argument('--reset', dest='reset', action='store_true',
                        help='reset database (clear all records)',)
    parser.set_defaults(host='0.0.0.0', port=5000,
                        database=database_location,
                        debug=False,
                        reset=False)

    args = parser.parse_args()

    norm_path = path.abspath(args.database)
    if not path.exists(path.dirname(norm_path)):
        os.makedirs(path.dirname(norm_path))

    if args.reset:
        os.remove(norm_path)

    app = create_app(database=norm_path)
    app.run(host=args.host, port=args.port, debug=args.debug)

if __name__ == '__main__':
    main()




from flask import Flask

from .database import db
from .models import *
from .views import reflectme

class WSGICopyBody(object):
    def __init__(self, application):
        self.application = application

    def __call__(self, environ, start_response):

        from cStringIO import StringIO
        length = environ.get('CONTENT_LENGTH', '0')
        length = 0 if length == '' else int(length)

        body = environ['wsgi.input'].read(length)
        environ['body_copy'] = body
        environ['wsgi.input'] = StringIO(body)

        # Call the wrapped application
        app_iter = self.application(environ,
                                    self._sr_callback(start_response))

        # Return modified response
        return app_iter

    def _sr_callback(self, start_response):
        def callback(status, headers, exc_info=None):

            # Call upstream start_response
            start_response(status, headers, exc_info)
        return callback

def create_app(database='database.db', testing=False):
    """Application factory"""

    app = Flask(__name__, instance_relative_config=True)
    app.testing = testing

    app.config.from_object('reflectme.settings')
    app.config.from_pyfile('settings.py', silent=True)

    database_uri = 'sqlite:///{0}'.format(database)
    app.config['SQLALCHEMY_DATABASE_URI'] = database_uri
    db.init_app(app)

    if not testing:
        with app.app_context():
            db.create_all()

    app.register_blueprint(reflectme)

    app.wsgi_app = WSGICopyBody(app.wsgi_app)
    return app



import json

from sqlalchemy.types import TypeDecorator, TEXT

from .database import db


class JSONEncodedDict(TypeDecorator):
    impl = TEXT

    def process_bind_param(self, value, dialect):
        if value is not None:
            value = json.dumps(value)

        return value

    def process_result_value(self, value, dialect):
        if value is not None:
            value = json.loads(value)
        return value


class Path(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    location = db.Column(db.String(255), index=True)
    status = db.Column(db.Integer)
    headers = db.Column(db.Text)
    response = db.Column(db.Text)
    records = db.relationship('Record', backref='path', lazy='joined', order_by="desc(Record.visited)")


class Record(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    path_id = db.Column(db.Integer, db.ForeignKey('path.id'))

    visited = db.Column(db.DateTime)
    request_ip = db.Column(db.String(255))
    request_content = db.Column(db.Text)




#!/usr/bin/env python

import os
import sys

try:
    from setuptools import setup, find_packages
except ImportError:
    from distutils.core import setup, find_packages


package_dir = os.path.join(os.path.dirname(os.path.abspath(__file__)))
with open(os.path.join(package_dir, 'requirements.txt')) as f:
    requires = [p.rstrip() for p in f.readlines()]


setup(
    name='reflectme',
    version='0.0.4',
    description='Simple HTTP server for recording requests and returning arbitrary responses.',
    author='Charles Lavery',
    author_email='charles.lavery@gmail.com',
    url='https://github.com/clavery/reflectme',
    packages=find_packages(),
    include_package_data=True,
    install_requires=requires,
    license='MIT License',
    zip_safe=False,
    classifiers=(
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7'
    ),
    entry_points = {
        'console_scripts': ['reflectme = reflectme.cli:main'],
    },
)
